from datetime import datetime

from sqlalchemy import Column, DateTime, Integer, String

from labelu.internal.common.db import Base


class User(Base):
    __tablename__ = "user"

    id = Column(Integer, primary_key=True, autoincrement=True, index=True)
    username = Column(String, unique=True, index=True)
    hashed_password = Column(String)
    created_at = Column(
        DateTime, default=datetime.utcnow, comment="Time a task was created"
    )
    updated_at = Column(
        DateTime, default=datetime.utcnow, comment="Last time a task was updated"
    )



from datetime import datetime

from typing import List, Union
from pydantic import BaseModel, Field

from labelu.internal.application.response.base import UserResp


class CreateSampleResponse(BaseModel):
    ids: Union[List[int], None] = Field(
        default=None, description="description: attachment ids"
    )


class SampleResponse(BaseModel):
    id: Union[int, None] = Field(default=None, description="description: annotation id")
    inner_id: Union[int, None] = Field(default=None, description="description: inner id of a sample in task")
    state: Union[str, None] = Field(
        default=None,
        description="description: sample file state, NEW is has not start yet, DONE is completed, SKIPPED is skipped",
    )
    data: Union[dict, None] = Field(
        default=None,
        description="description: sample data, include filename, file url, or result",
    )
    annotated_count: Union[int, None] = Field(
        default=0, description="description: annotate result count"
    )
    created_at: Union[datetime, None] = Field(
        default=None, description="description: task created at time"
    )
    created_by: Union[UserResp, None] = Field(
        default=None, description="description: task created by"
    )
    updated_at: Union[datetime, None] = Field(
        default=None, description="description: task updated at time"
    )
    updated_by: Union[UserResp, None] = Field(
        default=None, description="description: task updated by"
    )



from typing import Union
from pydantic import BaseModel, Field


class AttachmentResponse(BaseModel):
    id: Union[int, None] = Field(
        default=None, description="description: upload file id"
    )
    url: Union[str, None] = Field(
        default=None, description="description: upload file url"
    )



from fastapi import FastAPI

from labelu.internal.common.config import settings
from labelu.internal.middleware.tracing import TracingMiddleWare
from fastapi.middleware.trustedhost import TrustedHostMiddleware


def add_middleware(app: FastAPI):
    app.add_middleware(TracingMiddleWare)



from pydantic import BaseModel, EmailStr, Field


class SignupResponse(BaseModel):
    id: int
    username: EmailStr


class LoginResponse(BaseModel):
    token: str = Field(..., description="description: user credential")


class LogoutResponse(BaseModel):
    msg: str



from fastapi.encoders import jsonable_encoder
from sqlalchemy.orm import Session

from labelu.internal.domain.models.user import User
from labelu.internal.adapter.persistence import crud_user
from labelu.internal.common.security import verify_password
from labelu.internal.common.security import get_password_hash

from labelu.tests.utils.utils import random_username
from labelu.tests.utils.utils import random_lower_string


def test_create_user(db: Session) -> None:
    username = random_username()
    password = random_lower_string()
    user_in = User(username=username, hashed_password=get_password_hash(password))
    with db.begin():
        user = crud_user.create(db, user=user_in)
    assert user.username == username
    assert hasattr(user, "hashed_password")


def test_get_user(db: Session) -> None:
    password = random_lower_string()
    username = random_username()
    user_in = User(username=username, hashed_password=get_password_hash(password))
    with db.begin():
        user = crud_user.create(db, user=user_in)
    user_2 = crud_user.get(db, id=user.id)
    assert user_2
    assert user.username == user_2.username
    assert jsonable_encoder(user) == jsonable_encoder(user_2)


def test_get_user_by_username(db: Session) -> None:
    password = random_lower_string()
    username = random_username()
    user_in = User(username=username, hashed_password=get_password_hash(password))
    with db.begin():
        user = crud_user.create(db, user=user_in)
    user_2 = crud_user.get_user_by_username(db, username=user_in.username)
    assert user_2
    assert user.username == user_2.username
    assert jsonable_encoder(user) == jsonable_encoder(user_2)



"""Change Result format

Revision ID: 0145db0fec34
Revises: 363f9eea797e
Create Date: 2023-04-06 14:52:24.255060

"""
import json
from typing import Optional

from alembic import context, op
from pydantic import BaseModel
from sqlalchemy import update
from sqlalchemy.ext.automap import automap_base
from sqlalchemy.orm import sessionmaker

# revision identifiers, used by Alembic.
revision = "0145db0fec34"
down_revision = "363f9eea797e"
branch_labels = None
depends_on = None

# produce a set of mappings from this MetaData
Base = automap_base()

# define the new result model
class NewResult(BaseModel):
    x: Optional[float] = None
    y: Optional[float] = None
    width: Optional[float] = None
    height: Optional[float] = None
    attribute: Optional[str] = None
    valid: Optional[bool] = None
    isVisible: Optional[bool] = None
    id: Optional[str] = None
    sourceID: Optional[str] = None
    textAttribute: Optional[str] = None
    order: Optional[int] = None
    attributes: Optional[dict] = None
    pointList: Optional[list] = None
    label: Optional[str] = None
    isReference: Optional[bool] = None
    disableDelete: Optional[bool] = None
    isRect: Optional[bool] = None


# define the old result model
class OldResult(BaseModel):
    x: Optional[float] = None
    y: Optional[float] = None
    width: Optional[float] = None
    height: Optional[float] = None
    attribute: Optional[str] = None
    valid: Optional[bool] = None
    isVisible: Optional[bool] = None
    id: Optional[str] = None
    sourceID: Optional[str] = None
    textAttribute: Optional[str] = None
    order: Optional[int] = None
    result: Optional[dict] = None
    pointList: Optional[list] = None
    label: Optional[str] = None
    isReference: Optional[bool] = None
    disableDelete: Optional[bool] = None
    isRect: Optional[bool] = None

    def to_new(self):
        new_result = NewResult(
            x=self.x,
            y=self.y,
            width=self.width,
            height=self.height,
            attribute=self.attribute,
            valid=self.valid,
            isVisible=self.isVisible,
            id=self.id,
            sourceID=self.sourceID,
            textAttribute=self.textAttribute,
            order=self.order,
            pointList=self.pointList,
            label=self.label,
            isReference=self.isReference,
            disableDelete=self.disableDelete,
            isRect=self.isRect,
        )
        new_result.attributes = (
            self.result if self.result else {self.attribute: self.textAttribute}
        )
        return new_result


def upgrade() -> None:

    # get the current connection
    bind = op.get_bind()
    # Reflect ORM models from the database
    Base.prepare(autoload_with=bind, reflect=True)
    task_sample = Base.classes.task_sample
    # make a session
    Session = sessionmaker(bind=bind)
    session = Session()

    # replace the key with value in a transaction
    with context.begin_transaction():
        sample_items = session.execute(
            'SELECT id, data FROM task_sample WHERE data IS NOT NULL AND data != ""'
        )

        for sample_item in sample_items:

            sample_id = sample_item[0]
            sample_data = json.loads(sample_item[1])
            data_result = json.loads(sample_data.get("result", ""))
            for key in data_result.keys():
                if key.endswith("Tool"):
                    data_result[key]["result"] = [
                        OldResult.parse_obj(result).to_new().dict(exclude_none=True)
                        for result in data_result[key]["result"]
                    ]

            sample_data["result"] = json.dumps(data_result, ensure_ascii=False)
            data = json.dumps(sample_data, ensure_ascii=False)

            session.execute(
                update(task_sample).where(task_sample.id == sample_id).values(data=data)
            )


def downgrade() -> None:
    pass



from typing import Union

from pydantic import BaseModel
from typing import Generic, TypeVar
from typing import Generic, TypeVar
from pydantic.generics import GenericModel

DataT = TypeVar("DataT", bound=BaseModel)


class UserResp(BaseModel):
    id: Union[int, None]
    username: Union[str, None]


class CommonDataResp(BaseModel):
    ok: bool


class MetaData(BaseModel):
    total: int
    page: Union[int, None]
    size: int


class OkResp(GenericModel, Generic[DataT]):
    data: DataT


class OkRespWithMeta(GenericModel, Generic[DataT]):
    meta_data: Union[MetaData, None] = None
    data: DataT



import random
import string
from typing import Dict


def random_lower_string() -> str:
    return "".join(random.choices(string.ascii_lowercase, k=16))


def random_username() -> str:
    return f"{random_lower_string()}@{random_lower_string()}.com"



from typing import List, Union
from pydantic import BaseModel, Field
from fastapi import UploadFile, File

from labelu.internal.domain.models.task import MediaType


class AttachmentCommand(BaseModel):
    file: UploadFile = File()


class AttachmentDeleteCommand(BaseModel):
    attachment_ids: List[int] = Field(
        min_items=1,
        gt=0,
        description="description: attachment file id",
    )



from datetime import datetime

from sqlalchemy.schema import Index
from sqlalchemy import Column, DateTime, ForeignKey, Integer, SmallInteger, String

from labelu.internal.common.db import Base


class TaskAttachment(Base):
    __tablename__ = "task_attachment"

    id = Column(Integer, primary_key=True, autoincrement=True, index=True)
    path = Column(String(256), comment="file storage path")
    task_id = Column(Integer, ForeignKey("task.id"), index=True)
    created_by = Column(Integer, ForeignKey("user.id"), index=True)
    updated_by = Column(Integer, ForeignKey("user.id"), index=True)
    created_at = Column(
        DateTime, default=datetime.now, comment="Time a task was created"
    )
    updated_at = Column(
        DateTime,
        default=datetime.now,
        onupdate=datetime.now,
        comment="Last time a task was updated",
    )
    deleted_at = Column(DateTime, index=True, comment="Task delete time")

    Index("idx_attachment_id_deleted_at", id, deleted_at)



from typing import List, Union

from sqlalchemy.orm import Session
from fastapi import APIRouter, Depends, Query, status, Security
from fastapi.responses import FileResponse
from fastapi.security import HTTPAuthorizationCredentials

from labelu.internal.common import db
from labelu.internal.common.security import security
from labelu.internal.common.error_code import ErrorCode
from labelu.internal.common.error_code import LabelUException
from labelu.internal.domain.models.user import User
from labelu.internal.dependencies.user import get_current_user
from labelu.internal.application.service import sample as service
from labelu.internal.application.command.sample import ExportType
from labelu.internal.application.command.sample import PatchSampleCommand
from labelu.internal.application.command.sample import CreateSampleCommand
from labelu.internal.application.command.sample import DeleteSampleCommand
from labelu.internal.application.command.sample import ExportSampleCommand
from labelu.internal.application.response.base import OkResp
from labelu.internal.application.response.base import MetaData
from labelu.internal.application.response.base import CommonDataResp
from labelu.internal.application.response.base import OkRespWithMeta
from labelu.internal.application.response.sample import SampleResponse
from labelu.internal.application.response.sample import CreateSampleResponse


router = APIRouter(prefix="/tasks", tags=["samples"])


@router.post(
    "/{task_id}/samples",
    response_model=OkResp[CreateSampleResponse],
    status_code=status.HTTP_201_CREATED,
)
async def create(
    task_id: int,
    cmd: List[CreateSampleCommand],
    authorization: HTTPAuthorizationCredentials = Security(security),
    db: Session = Depends(db.get_db),
    current_user: User = Depends(get_current_user),
):
    """
    Create a sample.
    """

    # business logic
    data = await service.create(
        db=db, task_id=task_id, cmd=cmd, current_user=current_user
    )

    # response
    return OkResp[CreateSampleResponse](data=data)


@router.get(
    "/{task_id}/samples",
    response_model=OkRespWithMeta[List[SampleResponse]],
    status_code=status.HTTP_200_OK,
)
async def list_by(
    task_id: int,
    after: Union[int, None] = Query(default=None, gt=0),
    before: Union[int, None] = Query(default=None, gt=0),
    pageNo: Union[int, None] = Query(default=None, ge=0),
    pageSize: Union[int, None] = 100,
    sort: Union[str, None] = Query(
        default=None, regex="(annotated_count|state):(desc|asc)"
    ),
    authorization: HTTPAuthorizationCredentials = Security(security),
    db: Session = Depends(db.get_db),
    current_user: User = Depends(get_current_user),
):
    """
    Get a annotation result.
    """

    if len([i for i in (after, before, pageNo) if i != None]) != 1:
        raise LabelUException(
            status_code=status.HTTP_422_UNPROCESSABLE_ENTITY,
            code=ErrorCode.CODE_55000_SAMPLE_LIST_PARAMETERS_ERROR,
        )

    # business logic
    data, total = await service.list_by(
        db=db,
        task_id=task_id,
        after=after,
        before=before,
        pageNo=pageNo,
        pageSize=pageSize,
        sorting=sort,
        current_user=current_user,
    )

    # response
    meta_data = MetaData(total=total, page=pageNo, size=len(data))
    return OkRespWithMeta[List[SampleResponse]](meta_data=meta_data, data=data)


@router.get(
    "/{task_id}/samples/{sample_id}",
    response_model=OkResp[SampleResponse],
    status_code=status.HTTP_200_OK,
)
async def get(
    task_id: int,
    sample_id: int,
    authorization: HTTPAuthorizationCredentials = Security(security),
    db: Session = Depends(db.get_db),
    current_user: User = Depends(get_current_user),
):
    """
    Get a annotation result.
    """

    # business logic
    data = await service.get(
        db=db, task_id=task_id, sample_id=sample_id, current_user=current_user
    )

    # response
    return OkResp[SampleResponse](data=data)


@router.patch(
    "/{task_id}/samples/{sample_id}",
    response_model=OkResp[SampleResponse],
    status_code=status.HTTP_200_OK,
)
async def update(
    task_id: int,
    sample_id: int,
    cmd: PatchSampleCommand,
    authorization: HTTPAuthorizationCredentials = Security(security),
    db: Session = Depends(db.get_db),
    current_user: User = Depends(get_current_user),
):
    """
    update a annotation.
    """

    # business logic
    data = await service.patch(
        db=db, task_id=task_id, sample_id=sample_id, cmd=cmd, current_user=current_user
    )

    # response
    return OkResp[SampleResponse](data=data)


@router.delete(
    "/{task_id}/samples",
    response_model=OkResp[CommonDataResp],
    status_code=status.HTTP_200_OK,
)
async def delete(
    cmd: DeleteSampleCommand,
    authorization: HTTPAuthorizationCredentials = Security(security),
    db: Session = Depends(db.get_db),
    current_user: User = Depends(get_current_user),
):
    """
    delete a annotation.
    """

    # business logic
    data = await service.delete(
        db=db, sample_ids=cmd.sample_ids, current_user=current_user
    )

    # response
    return OkResp[CommonDataResp](data=data)


@router.post(
    "/{task_id}/samples/export",
    status_code=status.HTTP_200_OK,
)
async def export(
    task_id: int,
    export_type: ExportType,
    cmd: ExportSampleCommand,
    authorization: HTTPAuthorizationCredentials = Security(security),
    db: Session = Depends(db.get_db),
    current_user: User = Depends(get_current_user),
):
    """
    export data.
    """

    # business logic
    data = await service.export(
        db=db,
        task_id=task_id,
        export_type=export_type,
        sample_ids=cmd.sample_ids,
        current_user=current_user,
    )

    # response
    media_type = ".json" if data.suffix == ".json" else data.suffix.strip(".")
    return FileResponse(
        path=data, filename=data.name, media_type=f"application/{media_type}"
    )



from pydantic import BaseModel, EmailStr, Field


class SignupCommand(BaseModel):
    username: EmailStr
    password: str = Field(min_length=6, max_length=18)


class LoginCommand(BaseModel):
    username: EmailStr
    password: str = Field(min_length=6, max_length=18)



import uvicorn
from typer import Typer
from fastapi import FastAPI, Request
from fastapi.staticfiles import StaticFiles

from labelu.internal.adapter.routers import add_router
from labelu.internal.middleware import add_middleware
from labelu.internal.common.logger import init_logging
from labelu.internal.common.db import init_tables
from labelu.internal.common.config import settings
from labelu.internal.common.error_code import add_exception_handler
from labelu.alembic_labelu.run_migrate import run_sqlite_migrations


description = """
labelU backend.

## Users

You will be able to:

* **Signup**
* **Login**
* **Logout** (_not implemented_).

## Tasks

You will be able to:

* **CRUD**

## Task attachment

You will be able to:

* **upload attachment**
* **download attachment**
* **delete attachment**

## Task sample

You will be able to:

* **list sample**
* **create sample**
* **get sample**
* **update sample**
* **export sample**
"""


tags_metadata = [
    {
        "name": "users",
        "description": "Operations with users.",
    },
    {
        "name": "tasks",
        "description": "Task manangement.",
    },
    {
        "name": "attachments",
        "description": "Task attachment management.",
    },
    {
        "name": "samples",
        "description": "Task sample manangement.",
    },
]


app = FastAPI(
    title="labelU",
    description=description,
    version="0.1.0",
    terms_of_service="",
    contact={
        "name": "labelu",
        "url": "http://labelu.example.com/contact/",
        "email": "labelu@example.com",
    },
    license_info={
        "name": "Apache 2.0",
        "url": "https://www.apache.org/licenses/LICENSE-2.0.html",
    },
    openapi_tags=tags_metadata,
)

init_logging()
init_tables()
run_sqlite_migrations()
add_exception_handler(app=app)
add_router(app=app)
add_middleware(app=app)

app.mount("", StaticFiles(packages=["labelu.internal"], html=True))


@app.middleware("http")
async def add_correct_content_type(request: Request, call_next):
    response = await call_next(request)
    if request.url.path.endswith(".js"):
        response.headers["content-type"] = "application/javascript"
    return response


cli = Typer()


@cli.command()
def main(
    host: str = "localhost", port: int = 8000, media_host: str = "http://localhost:8000"
):
    if port:
        settings.PORT = port
    if host:
        settings.HOST = host
    if media_host:
        settings.MEDIA_HOST = media_host
    uvicorn.run(app=app, host=settings.HOST, port=settings.PORT)


if __name__ == "__main__":
    cli()



from enum import Enum
from datetime import datetime

from sqlalchemy.schema import Index
from sqlalchemy.orm import relationship
from sqlalchemy import Column, DateTime, ForeignKey, Integer, String, Text

from labelu.internal.common.db import Base


class SampleState(str, Enum):
    """
    task sample state
    """

    NEW = "NEW"
    SKIPPED = "SKIPPED"
    DONE = "DONE"


class TaskSample(Base):
    __tablename__ = "task_sample"

    id = Column(Integer, primary_key=True, autoincrement=True, index=True)
    inner_id = Column(Integer, comment="sample id in a task")
    task_id = Column(Integer, ForeignKey("task.id"), index=True)
    task_attachment_ids = Column(String(255), comment="task sample attachment ids")
    created_by = Column(Integer, ForeignKey("user.id"), index=True)
    updated_by = Column(Integer, ForeignKey("user.id"))
    created_at = Column(
        DateTime, default=datetime.now, comment="Time a task sample result was created"
    )
    updated_at = Column(
        DateTime,
        default=datetime.now,
        onupdate=datetime.now,
        comment="Last time a task sample result was updated",
    )
    annotated_count = Column(
        Integer,
        default=0,
        comment="task sample result count",
    )
    data = Column(Text, comment="task sample result")
    state = Column(
        String(32),
        default=SampleState.NEW.value,
        comment="NEW is has not start yet, DONE is completed, SKIPPED is skipped",
    )
    deleted_at = Column(DateTime, index=True, comment="Task delete time")

    task = relationship("Task", foreign_keys=[task_id])
    owner = relationship("User", foreign_keys=[created_by])
    updater = relationship("User", foreign_keys=[updated_by])

    Index("idx_sample_id_deleted_at", id, deleted_at)



from datetime import datetime

from typing import Union
from pydantic import BaseModel, EmailStr, Field

from labelu.internal.application.response.base import UserResp


class TaskResponse(BaseModel):
    id: Union[int, None] = Field(default=None, description="description: task id")
    name: Union[str, None] = Field(default=None, description="description: task name")
    description: Union[str, None] = Field(
        default=None, description="description: task description"
    )
    tips: Union[str, None] = Field(default=None, description="description: task tips")
    config: Union[str, None] = Field(
        default=None, description="description: task config content"
    )
    media_type: Union[str, None] = Field(
        default=None, description="description: task media type: IMAGE, VIDEO"
    )
    status: Union[str, None] = Field(
        default=None,
        description="description: task status: DRAFT, IMPORTED, CONFIGURED, INPROGRESS, FINISHED",
    )
    created_at: Union[datetime, None] = Field(
        default=None, description="description: task created at time"
    )
    created_by: Union[UserResp, None] = Field(
        default=None, description="description: task created by"
    )


class TaskStatics(BaseModel):
    new: Union[int, None] = Field(
        default=0, description="description: count for task data have not labeled yet"
    )
    done: Union[int, None] = Field(
        default=0, description="description: count for task data already labeled"
    )
    skipped: Union[int, None] = Field(
        default=0, description="description: count for task data skipped"
    )


class TaskResponseWithStatics(TaskResponse):
    stats: Union[TaskStatics, None] = None


class User(BaseModel):
    id: Union[int, None] = None
    username: Union[EmailStr, None] = None



from typing import Generator
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base
from labelu.internal.common.config import settings

# connect_args is needed only for SQLite. It's not needed for other databases
engine = create_engine(
    settings.DATABASE_URL, connect_args={"check_same_thread": False}, echo=True
)
SessionLocal = sessionmaker(autocommit=True, autoflush=False, bind=engine)

Base = declarative_base()

# create database tables
def init_tables() -> None:
    Base.metadata.create_all(bind=engine)


def get_db() -> Generator:
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()



from datetime import datetime
from typing import Any, Dict, List, Union


from sqlalchemy import case, func, text
from sqlalchemy.orm import Session
from fastapi.encoders import jsonable_encoder

from labelu.internal.domain.models.sample import SampleState
from labelu.internal.domain.models.sample import TaskSample


def batch(db: Session, samples: List[TaskSample]) -> List[TaskSample]:
    db.bulk_save_objects(samples, return_defaults=True)
    return samples


def list_by(
    db: Session,
    task_id: Union[int, None],
    owner_id: int,
    after: Union[int, None],
    before: Union[int, None],
    pageNo: Union[int, None],
    pageSize: int,
    sorting: Union[str, None],
) -> List[TaskSample]:

    # query filter
    query_filter = [TaskSample.created_by == owner_id, TaskSample.deleted_at == None]
    if before:
        query_filter.append(TaskSample.id < before)
    if after:
        query_filter.append(TaskSample.id > after)
    if task_id:
        query_filter.append(TaskSample.task_id == task_id)
    query = db.query(TaskSample).filter(*query_filter)

    # case when for state enum
    whens = {state: index for index, state in enumerate(SampleState)}
    sort_logic = case(value=TaskSample.state, whens=whens).label(TaskSample.state.key)

    if sorting:
        sort_strings = sorting.split(",")
        for item in sort_strings:
            sort_key = item.split(":")
            if sort_key[0] == TaskSample.state.key:
                if sort_key[1] == "asc":
                    query = query.order_by(sort_logic.asc())
                else:
                    query = query.order_by(sort_logic.desc())
            else:
                query = query.order_by(text(f"{sort_key[0]} {sort_key[1]}"))

    # default order by id, before need select last items
    if before:
        query = query.order_by(TaskSample.id.desc())
    else:
        query = query.order_by(TaskSample.id.asc())
    results = (
        query.offset(offset=pageNo * pageSize if pageNo else 0)
        .limit(limit=pageSize)
        .all()
    )
    if before:
        results.reverse()
    return results


def get(db: Session, sample_id: int) -> TaskSample:
    return (
        db.query(TaskSample)
        .filter(TaskSample.id == sample_id, TaskSample.deleted_at == None)
        .first()
    )


def get_by_ids(db: Session, sample_ids: List[int]) -> List[TaskSample]:
    return (
        db.query(TaskSample)
        .filter(TaskSample.id.in_(sample_ids), TaskSample.deleted_at == None)
        .all()
    )


def update(db: Session, db_obj: TaskSample, obj_in: Dict[str, Any]) -> TaskSample:
    obj_data = jsonable_encoder(obj_in)
    for field in obj_data:
        if field in obj_in:
            setattr(db_obj, field, obj_in[field])
    db.add(db_obj)
    db.flush()
    db.refresh(db_obj)
    return db_obj


def delete(db: Session, sample_ids: List[int]) -> None:
    db.query(TaskSample).filter(TaskSample.id.in_(sample_ids)).update(
        {TaskSample.deleted_at: datetime.now()}
    )


def count(db: Session, task_id: int, owner_id: int) -> int:
    query_filter = [TaskSample.created_by == owner_id, TaskSample.deleted_at == None]
    if task_id:
        query_filter.append(TaskSample.task_id == task_id)
    return db.query(TaskSample).filter(*query_filter).count()


def statics(
    db: Session,
    owner_id: int,
    task_ids: List[int],
) -> dict:
    query_filter = [TaskSample.created_by == owner_id, TaskSample.deleted_at == None]
    if task_ids:
        query_filter.append(TaskSample.task_id.in_(task_ids))

    statics = (
        db.query(
            TaskSample.task_id,
            TaskSample.state,
            func.count(TaskSample.state).label("count"),
        )
        .filter(*query_filter)
        .group_by(TaskSample.task_id, TaskSample.state)
        .all()
    )
    d = {f"{s[0]}_{s[1]}": s[2] for s in statics}
    return d



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['labelu',
 'labelu.alembic_labelu',
 'labelu.alembic_labelu.versions',
 'labelu.internal',
 'labelu.internal.adapter',
 'labelu.internal.adapter.persistence',
 'labelu.internal.adapter.routers',
 'labelu.internal.application',
 'labelu.internal.application.command',
 'labelu.internal.application.response',
 'labelu.internal.application.service',
 'labelu.internal.common',
 'labelu.internal.dependencies',
 'labelu.internal.domain.models',
 'labelu.internal.middleware',
 'labelu.internal.statics',
 'labelu.tests',
 'labelu.tests.internal',
 'labelu.tests.internal.adapter',
 'labelu.tests.internal.adapter.persistence',
 'labelu.tests.internal.adapter.routers',
 'labelu.tests.internal.common',
 'labelu.tests.utils']

package_data = \
{'': ['*'],
 'labelu.internal.statics': ['assets/*', 'src/icons/*', 'src/img/example/*'],
 'labelu.tests': ['data/*']}

install_requires = \
['aiofiles>=22.1.0,<23.0.0',
 'alembic>=1.9.4,<2.0.0',
 'appdirs>=1.4.4,<2.0.0',
 'email-validator>=1.3.0,<2.0.0',
 'fastapi>=0.86.0,<0.87.0',
 'loguru>=0.6.0,<0.7.0',
 'passlib[bcrypt]>=1.7.4,<2.0.0',
 'pillow>=9.3.0,<10.0.0',
 'python-dotenv>=0.21.0,<0.22.0',
 'python-jose[cryptography]>=3.3.0,<4.0.0',
 'python-multipart>=0.0.5,<0.0.6',
 'sqlalchemy>=1.4.43,<2.0.0',
 'typer[all]>=0.7.0,<0.8.0',
 'uvicorn>=0.19.0,<0.20.0']

entry_points = \
{'console_scripts': ['labelu = labelu.main:cli']}

setup_kwargs = {
    'name': 'labelu',
    'version': '0.9.3',
    'description': '',
    'long_description': '<div align="center">\n<article style="display: flex; flex-direction: column; align-items: center; justify-content: center;">\n    <p align="center"><img width="300" src="https://user-images.githubusercontent.com/25022954/209616423-9ab056be-5d62-4eeb-b91d-3b20f64cfcf8.svg" /></p>\n    <h1 style="width: 100%; text-align: center;"></h1>\n    <p align="center">\n        English | <a href="./README_zh-CN.md" >简体中文</a>\n    </p>\n</article>\n    \n   \n</div>\n\n## Introduction\n\nLabelU offers a variety of annotation tools and features, supporting image, video, and audio annotation.\n\n- Image: Multifunctional image processing tools encompassing 2D bounding box, cuboid, semantic segmentation, polylines, keypoints, and many other annotation tools, assist in completing image identification, annotation, and analysis.\n- Video: The video annotation has robust video processing capabilities, able to implement video segmentation, video classification, video information extraction, and other functions, providing high-quality annotated data for model training.\n- Audio: Highly efficient and accurate audio analysis tool can achieve audio segmentation, audio classification, audio information extraction, and other functions, making complex sound information visually intuitive.\n\n<p align="center">\n<img style="width: 600px" src="https://user-images.githubusercontent.com/25022954/209318236-79d3a5c3-2700-46c3-b59a-62d9c132a6c3.gif">\n</p>\n\n## Features\n\n- Simplicity: Provides a variety of image annotation tools that can be annotated through simple visual configuration.\n- Flexibility: A variety of tools can be freely combined to meet most image, video, and audio annotation needs.\n- Universality: Supports exporting to various data formats, including JSON, COCO, MASK.\n\n## Getting started\n\n- <a href="https://opendatalab.github.io/labelU-Kit/">\n    <button>Try LabelU annotation toolkit</button>\n</a>\n\n- <a href="https://labelu.shlab.tech/">\n    <button>Try LabelU online</button>\n</a>\n\n### Local deployment\n\n1. Install [Miniconda](https://docs.conda.io/en/latest/miniconda.html), Choose the corresponding operating system type and download it for installation.\n\n> **Note：** If your system is MacOS with an Intel chip, please install [Miniconda of intel x86_64](https://repo.anaconda.com/miniconda/)。\n\n2. After the installation is complete, run the following command in the terminal (you can choose the default \'y\' for prompts during the process):\n\n```bash\nconda create -n labelu python=3.7\n```\n\n> **Note：** For Windows platform, you can run the above command in Anaconda Prompt.\n\n3. Activate the environment：\n\n```bash\nconda activate labelu\n```\n\n4. Install LabelU：\n\n```bash\npip install labelu\n```\n\n> To install the test version：`pip install --extra-index-url https://test.pypi.org/simple/ labelu==<test revision>`\n\n5. Run LabelU：\n\n```bash\nlabelu\n```\n\n6. Visit [http://localhost:8000/](http://localhost:8000/) and ready to go.\n\n### Local development\n\n```bash\n# Download and Install miniconda\n# https://docs.conda.io/en/latest/miniconda.html\n\n# Create virtual environment(python = 3.7)\nconda create -n labelu python=3.7\n\n# Activate virtual environment\nconda activate labelu\n\n# Install peotry\n# https://python-poetry.org/docs/#installing-with-the-official-installer\n\n# Install all package dependencies\npoetry install\n\n# Start labelu, server: http://localhost:8000\nuvicorn labelu.main:app --reload\n\n# Update submodule\ngit submodule update --remote --merge\n```\n\n## Supported Scenarios\n\n### Image\n\n- Label Classification: Can help users quickly classify objects in images and can be used for image retrieval, object detection tasks.\n- Text Description: Text transcription can help users quickly extract text information in images and can be used for text retrieval, machine translation tasks.\n- Bounding Box: Can help users quickly select objects in images and can be used for image recognition, object tracking tasks.\n- Point Annotation: Points can help users accurately label key information in the image and can be used for object recognition, scene analysis tasks.\n- Polygon: Can help users accurately label irregular shapes and can be used for object recognition, scene analysis tasks.\n- Line Annotation: Lines can help users accurately label edges and contours in the image and can be used for object recognition, scene analysis tasks.\n- Cuboid: Cuboid can help users accurately label the size, shape, and location of objects within images, and can be used for object recognition, scene analysis tasks.\n\n### Video\n\n- Label Classification: Classifying and labeling videos can be used for video retrieval, recommendation, and classification tasks.\n- Text Description: Converting speech content in videos into text can be used for voice recognition, transcription, and translation tasks.\n- Segment Segmentation: Extracting specific clips or scenes from the video for annotation is very useful for video object detection, action recognition, and video summary tasks.\n- Timestamps: Point to or mark specific parts of the video; users can click on timestamps to jump directly to that part of the video.\n\n### Audio\n\n- Label Classification: By listening to the audio and selecting the appropriate classification for annotation, it\'s applicable for audio retrieval, recommendations, and classification tasks.\n- Text Description: Converting speech content in audio into text makes it easier for users to analyze and process text. It\'s very useful for voice recognition, transcription tasks, and can help users better understand and process voice content.\n- Segment Segmentation: Extracting specific clips from audio for annotation is very useful for audio event detection, voice recognition, and audio editing tasks.\n- Timestamps: Used to point to or mark specific parts of the audio; users can click on timestamps to jump directly to that part of the audio.\n\n## Quick start\n\n- [Guidance](https://opendatalab.github.io/labelU)\n\n## Annotation format\n\n- [Documentation](https://opendatalab.github.io/labelU/#/schema)\n\n## Communication\n\nWelcome to the OpenDataLab official WeChat group！\n\n<p align="center">\n<img style="width: 400px" src="https://user-images.githubusercontent.com/25022954/208374419-2dffb701-321a-4091-944d-5d913de79a15.jpg">\n</p>\n\n## Links\n\n- [LabelU-kit](https://github.com/opendatalab/labelU-Kit) (LabelU is developed using LabelU-kit.)\n\n## License\n\nThis project is released under the [Apache 2.0 license](./LICENSE).\n',
    'author': 'pengjinhu',
    'author_email': 'pengjinhu@pjlab.org.cn',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/opendatalab/labelU',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from datetime import timedelta

from sqlalchemy.orm import Session
from fastapi.testclient import TestClient

from labelu.internal.domain.models.user import User
from labelu.internal.adapter.persistence import crud_user
from labelu.internal.common.config import settings
from labelu.internal.common.security import AccessToken
from labelu.internal.common.security import get_password_hash
from labelu.internal.common.security import create_access_token

from labelu.tests.utils.utils import random_username
from labelu.tests.utils.utils import random_lower_string


class TestClassUserRouter:
    def test_user_signup_successful(self, client: TestClient, db: Session):

        # prepare data
        username = random_username()
        password = random_lower_string()
        data = {"username": username, "password": password}

        # run
        r = client.post(f"{settings.API_V1_STR}/users/signup", json=data)

        # check
        assert r.status_code == 201
        created_user = r.json()
        user = crud_user.get_user_by_username(db, username=username)
        assert user
        assert user.username == created_user["data"]["username"]

    def test_create_user_existing_username(self, client: TestClient, db: Session):
        # prepare data
        username = random_username()
        password = random_lower_string()
        data = {"username": username, "password": password}
        crud_user.create(
            db=db,
            user=User(username=username, hashed_password=get_password_hash(password)),
        )

        # run
        r = client.post(f"{settings.API_V1_STR}/users/signup", json=data)

        # check
        created_user = r.json()
        assert r.status_code == 400
        assert created_user["err_code"] == 40001

    def test_user_login_sucessful(self, client: TestClient, db: Session):
        # prepare data
        username = random_username()
        password = random_lower_string()
        data = {"username": username, "password": password}
        crud_user.create(
            db=db,
            user=User(username=username, hashed_password=get_password_hash(password)),
        )

        # run
        r = client.post(f"{settings.API_V1_STR}/users/login", json=data)

        # check
        assert r.status_code == 200
        assert r.json()["data"]["token"].startswith(settings.TOKEN_TYPE)

    def test_user_login_cannot_find_user(self, client: TestClient, db: Session):
        # prepare data
        username = random_username()
        password = random_lower_string()
        data = {"username": username, "password": password}

        # run
        r = client.post(f"{settings.API_V1_STR}/users/login", json=data)

        # check
        user = r.json()
        assert r.status_code == 401
        assert user["err_code"] == 40000

    def test_user_logout(
        self, client: TestClient, testuser_token_headers: dict, db: Session
    ):

        # run
        r = client.post(
            f"{settings.API_V1_STR}/users/logout", headers=testuser_token_headers
        )

        # check
        assert r.status_code == 200

    def test_user_token_error(self, client: TestClient, db: Session) -> None:

        # prepare data
        headers = {"Authorization": "Bearer token"}
        # run
        r = client.post(f"{settings.API_V1_STR}/users/logout", headers=headers)

        # check
        assert r.status_code == 403
        assert r.json()["err_code"] == 40003

    def test_cannot_not_found_token(self, client: TestClient, db: Session) -> None:

        # run
        r = client.post(f"{settings.API_V1_STR}/users/logout")

        # check
        assert r.status_code == 403
        assert r.json()["err_code"] == 30003

    def test_user_not_found(self, client: TestClient, db: Session) -> None:

        # prepare data
        username = random_username()
        access_token_expires = timedelta(minutes=settings.TOKEN_ACCESS_EXPIRE_MINUTES)
        access_token = create_access_token(
            token=AccessToken(id=-1, username=username),
            expires_delta=access_token_expires,
        )
        headers = {"Authorization": f"Bearer {access_token}"}
        # run
        r = client.post(f"{settings.API_V1_STR}/users/logout", headers=headers)

        # check
        assert r.status_code == 404
        assert r.json()["err_code"] == 40002



"""Change tool config format

Revision ID: 363f9eea797e
Revises: 9d5da133bbe4
Create Date: 2023-04-03 15:08:12.508189

"""
from random import choice
from typing import List, Optional

from alembic import context, op
from pydantic import BaseModel
from sqlalchemy import update
from sqlalchemy.ext.automap import automap_base
from sqlalchemy.orm import sessionmaker

# revision identifiers, used by Alembic.
revision = "363f9eea797e"
down_revision = "9d5da133bbe4"
branch_labels = None
depends_on = None

# produce a set of mappings from this MetaData
Base = automap_base()

# define the tool config default value
defaultColors = [
    "#fc5b50",
    "#FC7A42",
    "#FFB300",
    "#3BC141",
    "#50AAF2",
    "#506AFF",
    "#8D64FF",
    "#D55EEA",
    "#F974A4",
    "#DE8B3E",
    "#FF5C97",
    "#FFCC4A",
    "#F1F462",
    "#ABFF7E",
    "#52CEDD",
    "#5AB4DB",
    "#99D5F0",
    "#958FEC",
    "#AD6DDB",
    "#B77259",
]

stringTypeMap = {
    0: "text",
    1: "order",
    2: "english",
    3: "number",
    4: "regexp",
}

# define the new tool config model
class NewAttribute(BaseModel):
    color: Optional[str]
    key: Optional[str]
    value: Optional[str]


class NewOption(BaseModel):
    key: Optional[str]
    value: Optional[str]
    type: Optional[str]
    maxLength: Optional[int]
    stringType: Optional[str]
    defaultValue: Optional[str]
    regexp: Optional[str]
    isDefault: Optional[bool]


class NewConfigAttribute(BaseModel):
    color: Optional[str]
    key: Optional[str]
    value: Optional[str]
    options: Optional[List[NewOption]]
    attributes: Optional[List[NewOption]]
    type: Optional[str]
    stringType: Optional[str]
    required: Optional[bool]
    defaultValue: Optional[str]
    maxLength: Optional[int]


class NewConfig(BaseModel):
    isShowCursor: Optional[bool]
    showConfirm: Optional[bool]
    skipWhileNoDependencies: Optional[bool]
    drawOutsideTarget: Optional[bool]
    copyBackwardResult: Optional[bool]
    minWidth: Optional[int]
    attributeConfigurable: Optional[bool]
    textConfigurable: Optional[bool]
    attributes: Optional[List[NewConfigAttribute]]
    minHeight: Optional[int]
    edgeAdsorption: Optional[bool]
    lineType: Optional[int]
    isShowOrder: Optional[bool]
    upperLimit: Optional[str]
    lowerLimitPointNum: Optional[str]
    upperLimitPointNum: Optional[str]


class NewTool(BaseModel):
    tool: Optional[str]
    config: Optional[NewConfig]


class NewToolConfig(BaseModel):
    attributes: Optional[List[NewAttribute]]
    tools: Optional[List[NewTool]]
    commonAttributeConfigurable: Optional[bool]


# define the old tool config model
class OldAttributeItem(BaseModel):
    key: Optional[str]
    value: Optional[str]


class OldAttributeListItem(BaseModel):
    key: Optional[str]
    value: Optional[str]


class OldConfig(BaseModel):
    isShowCursor: Optional[bool]
    showConfirm: Optional[bool]
    skipWhileNoDependencies: Optional[bool]
    drawOutsideTarget: Optional[bool]
    copyBackwardResult: Optional[bool]
    minWidth: Optional[int]
    attributeConfigurable: Optional[bool]
    textConfigurable: Optional[bool]
    textCheckType: Optional[int]
    customFormat: Optional[str]
    attributeList: Optional[List[OldAttributeListItem]]
    minHeight: Optional[int]
    edgeAdsorption: Optional[bool]
    lineType: Optional[int]
    isShowOrder: Optional[bool]
    upperLimit: Optional[str]
    lowerLimitPointNum: Optional[str]
    upperLimitPointNum: Optional[str]


class OldTool(BaseModel):
    tool: Optional[str]
    config: Optional[OldConfig]


class OldSubSelectedItem(BaseModel):
    key: Optional[str]
    value: Optional[str]
    isDefault: Optional[bool]
    isMulti: Optional[bool]


class OldTagListItem(BaseModel):
    key: Optional[str]
    value: Optional[str]
    isMulti: Optional[bool]
    subSelected: Optional[List[OldSubSelectedItem]]


class OldTextConfigItem(BaseModel):
    label: Optional[str]
    key: Optional[str]
    required: Optional[bool]
    default: Optional[str]
    maxLength: Optional[int]


class OldToolConfig(BaseModel):
    attribute: Optional[List[OldAttributeItem]]
    tools: Optional[List[OldTool]]
    commonAttributeConfigurable: Optional[bool]
    tagList: Optional[List[OldTagListItem]]
    textConfig: Optional[List[OldTextConfigItem]]

    def to_new(self) -> NewToolConfig:

        new_tool_config = NewToolConfig()

        # attribute
        if self.attribute:

            new_tool_config.attributes = [
                NewAttribute(color=choice(defaultColors), **attribute.dict())
                for attribute in self.attribute
            ]

        # tools
        if self.tools:

            new_tool_config.tools = [
                NewTool(
                    tool=tool.tool,
                    config=NewConfig(
                        isShowCursor=tool.config.isShowCursor,
                        showConfirm=tool.config.showConfirm,
                        skipWhileNoDependencies=tool.config.skipWhileNoDependencies,
                        drawOutsideTarget=tool.config.drawOutsideTarget,
                        copyBackwardResult=tool.config.copyBackwardResult,
                        minWidth=tool.config.minWidth,
                        attributeConfigurable=tool.config.attributeConfigurable,
                        textConfigurable=tool.config.textConfigurable,
                        minHeight=tool.config.minHeight,
                        edgeAdsorption=tool.config.edgeAdsorption,
                        lineType=tool.config.lineType,
                        isShowOrder=tool.config.isShowOrder,
                        upperLimit=tool.config.upperLimit,
                        lowerLimitPointNum=tool.config.lowerLimitPointNum,
                        upperLimitPointNum=tool.config.upperLimitPointNum,
                        attributes=[
                            NewConfigAttribute(
                                color=choice(defaultColors),
                                attributes=[
                                    NewOption(
                                        key=attribute.key,
                                        value=attribute.value,
                                        type="string",
                                        maxLength=1000,
                                        stringType=stringTypeMap.get(
                                            tool.config.textCheckType, ""
                                        ),
                                        defaultValue="",
                                        regexp=tool.config.customFormat
                                        if tool.config.textCheckType == 4
                                        else None,
                                    )
                                ],
                                **attribute.dict(),
                            )
                            for attribute in tool.config.attributeList
                        ]
                        if tool.config.attributeList
                        else None,
                    )
                    if tool.config
                    else None,
                )
                for tool in self.tools
            ]

        # tagList
        if self.tagList:

            for index, tool in enumerate(new_tool_config.tools):

                if tool.tool == "tagTool":

                    new_tool_config.tools[index] = NewTool(
                        tool="tagTool",
                        config=NewConfig(
                            attributes=[
                                NewConfigAttribute(
                                    key=tag.key,
                                    value=tag.value,
                                    type="array" if tag.isMulti else "enum",
                                    options=[
                                        NewOption(
                                            key=subSelected.key,
                                            value=subSelected.value,
                                            isDefault=subSelected.isDefault
                                            if subSelected.isDefault != None
                                            else None,
                                        )
                                        for subSelected in tag.subSelected
                                    ]
                                    if tag.subSelected
                                    else None,
                                )
                                for tag in self.tagList
                            ],
                        ),
                    )

                    break

        # textConfig
        if self.textConfig:

            for index, tool in enumerate(new_tool_config.tools):

                if tool.tool == "textTool":

                    new_tool_config.tools[index] = NewTool(
                        tool="textTool",
                        config=NewConfig(
                            attributes=[
                                NewConfigAttribute(
                                    type="string",
                                    stringType="text",
                                    key=attribute_item.label,
                                    value=attribute_item.key,
                                    required=attribute_item.required,
                                    defaultValue=attribute_item.default,
                                    maxLength=attribute_item.maxLength,
                                )
                                for attribute_item in self.textConfig
                            ]
                        ),
                    )

                    break

        # commonAttributeConfigurable
        new_tool_config.commonAttributeConfigurable = self.commonAttributeConfigurable

        return new_tool_config


def upgrade() -> None:

    # get the current connection
    bind = op.get_bind()
    # Reflect ORM models from the database
    Base.prepare(autoload_with=bind, reflect=True)
    task = Base.classes.task
    # make a session
    Session = sessionmaker(bind=bind)
    session = Session()

    # replace the key with value in a transaction
    with context.begin_transaction():
        task_items = session.execute(
            'SELECT id, config FROM task WHERE config IS NOT NULL AND config != ""'
        )

        for task_item in task_items:
            task_id = task_item[0]
            old_tool_config = OldToolConfig.parse_raw(task_item[1])
            new_tool_config = old_tool_config.to_new()
            session.execute(
                update(task)
                .where(task.id == task_id)
                .values(
                    config=new_tool_config.json(exclude_none=True, ensure_ascii=False)
                )
            )


def downgrade() -> None:
    pass



"""Configure handlers and formats for application loggers."""
import logging
import sys
from pprint import pformat

# if you dont like imports of private modules
# you can move it to typing.py module
from loguru import logger
from loguru._defaults import LOGURU_FORMAT


class InterceptHandler(logging.Handler):
    """
    Default handler from examples in loguru documentaion.
    See https://loguru.readthedocs.io/en/stable/overview.html#entirely-compatible-with-standard-logging
    """

    def emit(self, record: logging.LogRecord):
        # Get corresponding Loguru level if it exists
        try:
            level = logger.level(record.levelname).name
        except ValueError:
            level = record.levelno

        # Find caller from where originated the logged message
        frame, depth = logging.currentframe(), 2
        while frame.f_code.co_filename == logging.__file__:
            frame = frame.f_back
            depth += 1

        logger.opt(depth=depth, exception=record.exc_info).log(
            level, record.getMessage()
        )


def format_record(record: dict) -> str:
    """
    Custom format for loguru loggers.
    Uses pformat for log any data like request/response body during debug.
    Works with logging if loguru handler it.

    Example:
    >>> payload = [{"users":[{"name": "Nick", "age": 87, "is_active": True}, {"name": "Alex", "age": 27, "is_active": True}], "count": 2}]
    >>> logger.bind(payload=).debug("users payload")
    >>> [   {   'count': 2,
    >>>         'users': [   {'age': 87, 'is_active': True, 'name': 'Nick'},
    >>>                      {'age': 27, 'is_active': True, 'name': 'Alex'}]}]
    """

    format_string = LOGURU_FORMAT
    if record["extra"].get("payload") is not None:
        record["extra"]["payload"] = pformat(
            record["extra"]["payload"], indent=4, compact=True, width=88
        )
        format_string += "\n<level>{extra[payload]}</level>"

    format_string += "{exception}\n"
    return format_string


def init_logging():
    """
    Replaces logging handlers with a handler for using the custom handler.

    WARNING!
    if you call the init_logging in startup event function,
    then the first logs before the application start will be in the old format

    >>> app.add_event_handler("startup", init_logging)
    stdout:
    INFO:     Uvicorn running on http://127.0.0.1:8000 (Press CTRL+C to quit)
    INFO:     Started reloader process [11528] using statreload
    INFO:     Started server process [6036]
    INFO:     Waiting for application startup.
    2020-07-25 02:19:21.357 | INFO     | uvicorn.lifespan.on:startup:34 - Application startup complete.

    """

    # disable handlers for specific uvicorn loggers
    # to redirect their output to the default uvicorn logger
    # works with uvicorn==0.11.6
    loggers = (logging.getLogger(name) for name in logging.root.manager.loggerDict)
    for l in loggers:
        l.handlers = []

    # change handler for default uvicorn logger
    intercept_handler = InterceptHandler()
    log_names = [
        "fastapi",
        "uvicorn",
        "uvicorn.access",
        "sqlalchemy",
        "sqlalchemy.pool",
    ]
    for log_name in log_names:
        logging.getLogger(log_name).handlers = [intercept_handler]

    # set logs output, level and format
    logger.configure(
        handlers=[{"sink": sys.stdout, "level": logging.DEBUG, "format": format_record}]
    )



from typing import Optional

from jose import jwt
from sqlalchemy.orm import Session
from pydantic import ValidationError
from fastapi import status, Depends, Request

from labelu.internal.domain.models.user import User
from labelu.internal.common import db
from labelu.internal.common.config import settings
from labelu.internal.common.security import AccessToken
from labelu.internal.common.error_code import ErrorCode
from labelu.internal.common.error_code import LabelUException
from labelu.internal.adapter.persistence import crud_user


class SpecialOAuth2PasswordBearer:
    def __call__(self, request: Request) -> Optional[str]:
        authorization: str = request.headers.get("Authorization")
        _, _, param = authorization.partition(" ")
        return param


reusable_oauth2 = SpecialOAuth2PasswordBearer()


def get_current_user(
    db: Session = Depends(db.get_db), token: str = Depends(reusable_oauth2)
) -> User:
    try:
        payload = jwt.decode(
            token,
            settings.PASSWORD_SECRET_KEY,
            algorithms=[settings.TOKEN_GENERATE_ALGORITHM],
            options={"verify_exp": False},
        )
        token_data = AccessToken(**payload)
    except (jwt.JWTError, ValidationError):
        raise LabelUException(
            code=ErrorCode.CODE_40003_CREDENTIAL_ERROR,
            status_code=status.HTTP_403_FORBIDDEN,
        )
    user = crud_user.get(db, id=token_data.id)
    if not user:
        raise LabelUException(code=ErrorCode.CODE_40002_USER_NOT_FOUND, status_code=404)
    return user
